/* 

Copyright (C) 2003-2004 Yves Schmid (www.garagecube.com)

Author(s): Yves Schmid

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     GNU General Public License for more details.

You should have received a copy of the GNU General Public Licensealong with this program; if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#import "CosmicDocument.h"
#import "CosmicOpenGLView.h"

#include "OMediaMemStream.h"
#include "OMediaMoveableMem.h"
#include "OMediaCanvas.h"
#include "OMediaViewport.h"

@implementation CosmicDocument

- (id)init
{
    self = [super init];
    if (self) 
	{
		setRotateSpeed = 0.9;
		setFPSFactor = 1.0;

		setPX = 0;
		setPY = 0;
		setPZ = 0;

		setAX = 0;
		setAY = 0;
		setAZ = 0;
	
		setScrollSpeed = 0;

		setScaleSpeed = 0;
		setScaleStart = 0;
		setScaleEnd = 1;
		setScalePingpong = true;

		setCanvas = NULL;

    }
    return self;
}

-(void)dealloc
{
	delete setCanvas;
	[super dealloc];
}

-(void)close
{
	[mainView stopAndReleaseAll];

	[super close];
}

- (NSString *)windowNibName
{
    return @"CosmicDocument";
}

- (void)windowControllerDidLoadNib:(NSWindowController *) aController
{
    [super windowControllerDidLoadNib:aController];
}

- (NSData *)dataRepresentationOfType:(NSString *)aType
{
    OMediaMemStream		stream;
    OMediaMoveableMem	*mb;
    NSData				*data;
	unsigned long		fileid = 'COSP';
	long				version = 3,size;
	OMediaMemStream		imgmemstream;
	
	if (!mainView || !mainView->canvas) return NULL;

	[mainView lock];

	stream<<fileid;
	stream<<version;
	stream<<mainView->rotationSpeed;
	stream<<mainView->fpsFactor;

	// V3 (2.0)

	float x,y,z;
	
	x = mainView->viewport->getx();
	y = mainView->viewport->gety();
	z = mainView->viewport->getz();

	stream<<x;
	stream<<y;
	stream<<z;

	x = mainView->viewport->get_anglex();
	y = mainView->viewport->get_angley();
	z = mainView->viewport->get_anglez();

	stream<<x;
	stream<<y;
	stream<<z;
	
	stream<<mainView->scrollSpeed;

	stream<<mainView->scaleSpeed;
	stream<<mainView->scaleStart;
	stream<<mainView->scaleEnd;
	stream<<mainView->scalePingpong;

	// Save image as a compressed png

	mainView->canvas->png_export(imgmemstream);
    mb = imgmemstream.get_memory_block();
    size = mb->getsize();
	stream<<size;
	stream.write(mb->lock(),size);
    mb->unlock();
   
    mb = stream.get_memory_block();
    size = mb->getsize();
    
    data = [NSData dataWithBytes:mb->lock() length:size];
    mb->unlock();

	[mainView unlock];

	return data;
}

- (BOOL)loadDataRepresentation:(NSData *)data ofType:(NSString *)aType
{
    OMediaMemStream		stream;
    OMediaMoveableMem	*mb;
	long				size;
	unsigned long		fileid;
	long				version;

	try
	{
		[mainView lock];
		delete setCanvas;
		setCanvas = new OMediaCanvas;
	
		size = [data length];
		stream.setsize(size);
		mb = stream.get_memory_block();

		memcpy(mb->lock(),[data bytes],size);
		mb->unlock();
		
		stream>>fileid;
		if (fileid!='COSP') {[mainView unlock];return NO;}
		stream>>version;
		if (version>3) {[mainView unlock];return NO;}

		stream>>setRotateSpeed;
		if (version>=2) stream>>setFPSFactor;
		else setFPSFactor = 1.0;

		if (version>=3)
		{
			stream>>setPX;
			stream>>setPY;
			stream>>setPZ;

			stream>>setAX;
			stream>>setAY;
			stream>>setAZ;
			
			stream>>setScrollSpeed;
		
			stream>>setScaleSpeed;
			stream>>setScaleStart;
			stream>>setScaleEnd;
			stream>>setScalePingpong;
		}
		
		if (version<1) stream>>setCanvas;		
		else
		{
			stream>>size;
			stream>>setCanvas;
		}

		[mainView unlock];
    }
    catch(...){[mainView unlock];return NO;}

    return YES;
}

-(IBAction)updateInfoText:(id)sender
{
	NSRect  r = [mainView bounds];
	int		dps,fps;
	
	fps = (int)(1.0/(mainView->wantedRate * (1.0/mainView->fpsFactor)));
	dps = (int)((omd_Angle2DegF(mainView->rotationSpeed))*1000.0);

	[infoTextField setStringValue:
			[NSString stringWithFormat:@"Size: %d      Degrees Per Sec.: %d    Frames Per Sec.: %d",(int)r.size.width,dps,fps]];
}



@end
